Files
derek.holloway 7502bf64a4
CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
Flatten subprojects into a single base monorepo
2026-09-02 16:34:47 -07:00

103 lines
3.9 KiB
C++

// MIT License
//
// Copyright (c) 2026 Advanced Micro Devices, Inc. All rights reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.
#include <hip/hip_runtime.h>
#include <atomic>
#include <chrono>
#include <iostream>
#include <thread>
#define HIP_CALL(call) \
do { \
hipError_t err = call; \
if (err != hipSuccess) \
{ \
fprintf(stderr, "%s\n", hipGetErrorString(err)); \
abort(); \
} \
} \
while (0)
__device__ void sync() { __syncthreads(); }
// A small device function that should NOT be instrumented
__device__ float add_one(float x)
{
asm volatile("v_add_f32 %0, %1, 1" : "=v"(x) : "v"(x));
asm volatile("v_add_f32 %0, %1, 1" : "=v"(x) : "v"(x));
asm volatile("v_add_f32 %0, %1, 1" : "=v"(x) : "v"(x));
asm volatile("v_add_f32 %0, %1, 1" : "=v"(x) : "v"(x));
asm volatile("s_nop 3");
return x;
}
__device__ float heavy_compute(int iters, float* out)
{
float result = out[threadIdx.x];
for (int i = 0; i < iters; i++)
{
result = add_one(result);
result = result / (result + 1.0f);
result = add_one(result);
result = result * 2.0f - 0.5f;
if (i % 8 == 0)
{
sync();
out[(threadIdx.x + 64) % 256] += result;
sync();
}
else
result += out[threadIdx.x] * 0.01f;
}
return result;
}
__global__ void compute_kernel(float* out, const float* in, int size, int iters)
{
int idx = blockIdx.x * blockDim.x + threadIdx.x;
if (idx >= size) return;
__shared__ float shm[256];
shm[threadIdx.x] = in[idx];
float val = heavy_compute(iters, shm);
out[idx] = add_one(val);
}
int main()
{
const int N = 304 * 256;
float *d_in, *d_out;
hipMalloc(&d_in, N * sizeof(float));
hipMalloc(&d_out, N * sizeof(float));
std::vector<float> h_in(N, 1.0f);
hipMemcpy(d_in, h_in.data(), N * sizeof(float), hipMemcpyHostToDevice);
compute_kernel<<<(N + 255) / 256, 256>>>(d_out, d_in, N, 10);
hipDeviceSynchronize();
hipFree(d_in);
hipFree(d_out);
return 0;
}